/*
 
 *************************************************************************
 
 Armagetron Printer Interface
 Developed by Morgan Jones (maclover201@me.com)
 
 **************************************************************************
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 ***************************************************************************
 
 */

#include "config.h"

ConfigItemBase::ConfigItemMap *ConfigItemBase::ConfMap = NULL;

ConfigItemBase::ConfigItemMap & ConfigItemBase::MakeConfMap()
{
	if( !ConfMap )
		ConfMap = new ConfigItemMap;
	return *ConfMap;
}

void ConfigItemBase::WriteDefaultContents(string path)
{
	ofstream file;
	file.open( path.c_str(), ofstream::trunc );
	if( !file.is_open() || !file.good() )
	{
		stringstream err;
		err << "Error creating file \"" << path << ".\"";
		Log::die( err.str(), 5 );
	}
	file
	<< "# Configuration file automatically generated by Armagetron Advanced Printer Interface" << endl 
	<< "# Syntax of this file is similar to the Armagetron Advanced configuration files" << endl
	<< "# Number signs (#) represent comments, as well as semicolons (;)" << endl
	<< "# They act the same way and are based on personal preference." << endl
	<< "# Setting names can be in ALL CAPS, all lowercase, or even in mIxEd cAsE." << endl
	<< endl
	<< "; General Settings" << endl
	<< endl
	<< "# What is your printer's IP address? Make sure to set this or you'll get an error." << endl
	<< "PRINTER_IP 0.0.0.0" << endl
	<< endl
	<< "# What are the names of the servers you want the program to monitor? Separate them with semicolons." << endl
	<< "# Example: internalDrive -=- iD Tech Camps;internalDrive -=- iD Tech Racing" << endl
	<< "SERVER_NAMES NONE" << endl
	<< endl
	<< "# What are the locations of the players.txt files that correspond to the server names? Again, separate them" << endl
	<< "# with semicolons." << endl
	<< "# Example: /var/www/internaldrive.info/logs/normal/players.txt;/var/www/internaldrive.info/logs/race/players.txt" << endl
	<< "PLAYER_FILES NONE" << endl
	<< endl
	<< "; Fine Tuning" << endl
	<< endl
	<< "# How many characters can your printer display at once?" << endl
	<< "DISPLAY_MAX 16" << endl
	<< endl
	<< "# What scroll type do you want? SCROLL_TYPE 0 makes the text be displayed in blocks" << endl
	<< "# (DISPLAY_MAX - 2) characters long, terminated by a '>>' sign that indicates that" << endl
	<< "# there is more text to display, with pauses of INTERMEDIATE_PAUSE seconds and FINAL_PAUSE seconds" << endl
	<< "# at the last block. SCROLL_TYPE 1 makes the text be displayed ticker-style, with a pause" << endl
	<< "# of INITIAL_PAUSE seconds before it scrolls. SCROLL_TYPE 2 makes the text be displayed in" << endl
	<< "# a pseudo-ticker style. It displays the text, then waits INITIAL_PAUSE seconds before" << endl
	<< "# scrolling across the screen, then it waits FINAL_PAUSE seconds before moving on to the next" << endl
	<< "# message." << endl
	<< "SCROLL_TYPE 0" << endl
	<< endl
	<< "# How fast (in seconds) do you want the characters to scroll across the display?" << endl
	<< "# Good values are greater than or equal to 0.2." << endl
	<< "SCROLL_RATE 0.2" << endl
	<< endl
	<< "# How long (in seconds) do you want the scroll to pause before the text starts scrolling?" << endl
	<< "# NOTE: This only works with SCROLL_TYPE 1 and SCROLL_TYPE 2." << endl
	<< "INITIAL_PAUSE 1.85" << endl
	<< endl
	<< "# How long (in seconds) do you want the scroll to pause between blocks of text?" << endl
	<< "# NOTE: This only works with SCROLL_TYPE 0." << endl
	<< "INTERMEDIATE_PAUSE 2.25" << endl
	<< endl
	<< "# How long (in seconds) do you want the scroll to pause after the text finishes scrolling?" << endl
	<< "# NOTE: This only works with SCROLL_TYPE 0 and SCROLL_TYPE 2." << endl
	<< "FINAL_PAUSE 3" << endl
	<< endl
	<< "# How long (in seconds) do you want the text to pause between introductory displays?" << endl
	<< "# These displays are the \"Armagetron,\" \"Printer,\" and \"Interface\" messages" << endl
	<< "# displayed on the printer when the program loads. The version will remain on the screen" << endl
	<< "# for (INTRO_PAUSE * 2.15) seconds." << endl
	<< "INTRO_PAUSE 2" << endl << endl;
	file.close();
}

vector <string> ConfigItemBase::Parse(string toParse)
{
	vector <string> buffer;
	size_t pos;
	
	// strip any excess whitespace off the beginning and end of the config item
	for( string::iterator it = toParse.begin(); ( *it == ' ' || *it == '\t' || *it == '\n' ) && it < toParse.end(); it++ )
	{
		toParse.erase( it - 1 );
	}
	
	for( string::reverse_iterator rit = toParse.rbegin(); ( *rit == ' ' || *rit == '\t' || *rit == '\n' ) && rit < toParse.rend(); rit++ )
	{
		toParse.erase( rit.base() - 1 );
	}
	
	pos = toParse.find_first_of( " \t\n" );
	string temp = toParse.substr( 0, pos );
	
	// make it caps
	for( int a = 0; a < temp.length(); a++ )
	{
		temp[a] = toupper( temp[a] );
	}
	
	buffer.push_back( temp );
	
	pos = toParse.find_first_not_of( " \t\n", pos + 1 );
	
	buffer.push_back( toParse.substr( pos ) );
	
	return buffer;
}

vector <string> ConfigItemBase::LoadFile(string path)
{
	ifstream file;
	
	file.open( path.c_str(), ifstream::in );
	
	if( !file.is_open() || !file.good() )
	{
		WriteDefaultContents( path );
		file.open( path.c_str(), ifstream::in );
	}
	
	vector <string> contents;
	
	while( !file.eof() && file.good() )
	{
		string temp;
		getline( file, temp );
		if( *temp.begin() != '#' && *temp.begin() != ';' && !temp.empty() ) // no comments or empty lines
			contents.push_back( temp );
	}
	file.close();
	return contents;
}