/*
 
 *************************************************************************
 
 Armagetron Printer Interface
 Developed by Morgan Jones (maclover201@me.com)
 
 **************************************************************************
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 ***************************************************************************
 
 */

#pragma once

#ifndef CONFIG_H
#define CONFIG_H

#include <string.h>
#include <map>
#include <new>
#include <vector>
#include <fstream>
#include <sstream>
#include <iostream>

#include "defs.h"

#include "log.h"

static bool Initialized = false;

typedef float REAL;

class ConfigItemBase {
protected:
	typedef map <string, string> ConfigItemMap;
	static ConfigItemMap *ConfMap;
	static ConfigItemMap &MakeConfMap();
public:
	static void WriteDefaultContents(string path);
	static vector <string> Parse(string toParse);
	static vector <string> LoadFile(string path);
	virtual ~ConfigItemBase() {}
};

template <class T> class ConfigItem : public ConfigItemBase {
public:
	ConfigItem(string name, T &var)
	{
		ConfigItemBase::ConfigItemMap & myMap = ConfigItemBase::MakeConfMap();
		
		if( !Initialized )
		{
			Path::SetExecutableDirectory();
			vector <string> data = ConfigItemBase::LoadFile( "settings.cfg" );
			for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
			{
				pair <map <string, string>::iterator, bool > ret;
				vector <string> parsed = ConfigItemBase::Parse( *it );
				ret = myMap.insert( pair <string, string> ( parsed[0], parsed[1] ) );
				if( !ret.second )
				{
					stringstream warning;
					warning << "Setting item " << parsed[0] << " already exists with a value of " << ret.first->second << ". Using original value." << endl;
					Log::warn( warning.str() );
				}
			}
			Initialized = true;
		}
		if( myMap.count( name ) > 0 )
			var = myMap[name];
	}
	virtual ~ConfigItem() {}
};

template <> class ConfigItem <string> : public ConfigItemBase {
public:
	ConfigItem(string name, string &var)
	{
		ConfigItemBase::ConfigItemMap & myMap = ConfigItemBase::MakeConfMap();
		
		if( !Initialized )
		{
			vector <string> data = ConfigItemBase::LoadFile( "settings.cfg" );
			for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
			{
				pair <map <string, string>::iterator, bool > ret;
				vector <string> parsed = ConfigItemBase::Parse( *it );
				ret = myMap.insert( pair <string, string> ( parsed[0], parsed[1] ) );
				if( !ret.second )
				{
					stringstream warning;
					warning << "Setting item " << parsed[0] << " already exists with a value of " << ret.first->second << ". Using original value." << endl;
					Log::warn( warning.str() );
				}
			}
			Initialized = true;
		}
		if( myMap.count( name ) > 0 )
			var = myMap[name];
	}
	virtual ~ConfigItem() {}
};

template <> class ConfigItem <int> : public ConfigItemBase {
public:
	ConfigItem(string name, int &var)
	{
		ConfigItemBase::ConfigItemMap & myMap = ConfigItemBase::MakeConfMap();
		
		if( !Initialized )
		{
			vector <string> data = ConfigItemBase::LoadFile( "settings.cfg" );
			for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
			{
				pair <map <string, string>::iterator, bool > ret;
				vector <string> parsed = ConfigItemBase::Parse( *it );
				ret = myMap.insert( pair <string, string> ( parsed[0], parsed[1] ) );
				if( !ret.second )
				{
					stringstream warning;
					warning << "Setting item " << parsed[0] << " already exists with a value of " << ret.first->second << ". Using original value." << endl;
					Log::warn( warning.str() );
				}
			}
			Initialized = true;
		}
		if( myMap.count( name ) > 0 )
		{
			int i = atoi( myMap[name].c_str() );
			if( i == 0 && myMap[name] != "0" )
			{
				stringstream warning;
				warning << "Invalid value for setting item " << name << ", using default of " << var << "." << endl;
				Log::warn( warning.str() );
			}
			else
				var = i;
		}
	}
	virtual ~ConfigItem() {}
};

template <> class ConfigItem <REAL> : public ConfigItemBase {
public:
	ConfigItem(string name, REAL &var)
	{
		ConfigItemBase::ConfigItemMap & myMap = ConfigItemBase::MakeConfMap();
		
		if( !Initialized )
		{
			vector <string> data = ConfigItemBase::LoadFile( "settings.cfg" );
			for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
			{
				pair <map <string, string>::iterator, bool > ret;
				vector <string> parsed = ConfigItemBase::Parse( *it );
				ret = myMap.insert( pair <string, string> ( parsed[0], parsed[1] ) );
				if( !ret.second )
				{
					stringstream warning;
					warning << "Setting item " << parsed[0] << " already exists with a value of " << ret.first->second << ". Using original value." << endl;
					Log::warn( warning.str() );
				}
			}
			Initialized = true;
		}
		if( myMap.count( name ) > 0 )
		{
			REAL i = static_cast <REAL> ( strtod( myMap[name].c_str(), NULL ) );
			if( i == 0.0f && ( myMap[name] != "0" || myMap[name] != "0.0" || myMap[name] != "0.0f" ) )
			{
				stringstream warning;
				warning << "Invalid value for setting item " << name << ", using default of " << var << "." << endl;
				Log::warn( warning.str() );
			}
			else
				var = i;
		}
	}
	virtual ~ConfigItem() {}
};

template <> class ConfigItem <bool> : public ConfigItemBase {
public:
	ConfigItem(string name, bool &var)
	{
		ConfigItemBase::ConfigItemMap & myMap = ConfigItemBase::MakeConfMap();
		
		if( !Initialized )
		{
			vector <string> data = ConfigItemBase::LoadFile( "settings.cfg" );
			for( vector <string>::iterator it = data.begin(); it != data.end(); it++ )
			{
				pair <map <string, string>::iterator, bool > ret;
				vector <string> parsed = ConfigItemBase::Parse( *it );
				ret = myMap.insert( pair <string, string> ( parsed[0], parsed[1] ) );
				if( !ret.second )
				{
					stringstream warning;
					warning << "Setting item " << parsed[0] << " already exists with a value of " << ret.first->second << ". Using original value." << endl;
					Log::warn( warning.str() );
				}
			}
			Initialized = true;
		}
		if( myMap.count( name ) > 0 )
		{
			if( myMap[name] == "1" || myMap[name] == "true" || myMap[name] == "yes" || myMap[name] == "on" )
				var = true;
			else if( myMap[name] == "0" || myMap[name] == "false" || myMap[name] == "no" || myMap[name] == "off" )
				var = false;
			else
			{
				stringstream warning;
				warning << "Invalid value for setting item " << name << ", using default of " << var << "." << endl;
				Log::warn( warning.str() );
			}
		}
	}
	virtual ~ConfigItem() {}
};

#endif // CONFIG_H