/*
 
 *************************************************************************
 
 Armagetron Printer Interface
 Developed by Morgan Jones (maclover201@me.com)
 
 **************************************************************************
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 ***************************************************************************
 
 */

#include "printer.h"

#include "socket.h"
#include "log.h"

// From main.cpp
extern int display_max;
extern int scroll_type;
extern REAL scroll_rate;
extern REAL initial_pause;
extern REAL intermediate_pause;
extern REAL final_pause;

void Printer::Display(string message, bool scroll)
{
	if( message.empty() )
	{
		stringstream err;
		err << "Oops! Message is empty! (file " << __FILE__ << ", line #" << __LINE__ << ')' << endl
		<< "Please email maclover201@me.com with this information. Thank you." << endl;
		
		Printer::Reset();
		Log::die( err.str(), 255 );
	}
	
	if( message.length() <= display_max || scroll == false )
	{
		Socket::Write( message );
		return;
	}
	if( message.length() > display_max && scroll == true )
	{
		if( scroll_type <= 0 )
		{
			for( size_t pos = 0; pos <= message.length(); pos += display_max - 2 )
			{
				string str( message.substr( pos, display_max - 2) );
				if( pos + display_max <= message.size() ) // <message> »
					str.append( " \xFD" );
				Socket::Write( str );
				wait( intermediate_pause );
			}
			wait( final_pause - intermediate_pause );
		}
		if( scroll_type == 1 )
		{
			Socket::Write( message.substr( 0, display_max ) ); // substr takes care of the out-of-bounds issue for us
			wait( initial_pause );
			for( size_t a = 1; a < message.length() + 1; a++ )
			{
				Socket::Write( message.substr( a, display_max ) ); // substr takes care of the out-of-bounds issue for us
				wait( scroll_rate );
			}
		}
		if( scroll_type >= 2 )
		{
			Socket::Write( message.substr( 0, display_max ) ); // substr takes care of the out-of-bounds issue for us
			wait( initial_pause );
			for( size_t a = 1; a < (message.length() + 1) - display_max; a++ )
			{
				Socket::Write( message.substr( a, display_max ) );
				wait( scroll_rate );
			}
			wait( final_pause - scroll_rate );
		}
		return;
	}
}

void Printer::Reset()
{
	Log::info( "Resetting printer.\n" );
	Socket::Write( "" );
}